import { Observable } from 'rxjs';
import { Nullable, Try } from 'javascriptutilities';
import { State as S } from 'type-safe-state-js';
import { Type as StoreType } from './types';
export declare namespace action {
    /**
     * Represents a dispatchable action.
     * @template T Generics parameter.
     */
    interface Type<T> {
        readonly name: string;
        readonly value: T;
    }
}
export declare namespace stateinfo {
    /**
     * Represents necessary state information.
     * @template T Generics parameter.
     */
    interface Type<T> {
        readonly state: S.Type<T>;
        readonly lastAction: Nullable<action.Type<T>>;
    }
}
export declare type ActionType<T> = action.Type<T> | T;
export declare type Reducer<T> = (state: S.Type<T>, action: action.Type<T>) => S.Type<T>;
export declare type RxReducer<T> = (state: S.Type<T>) => stateinfo.Type<T>;
/**
 * Create reducer streams that can be used to calculate states. Beware that the
 * resulting Observable emits a function that takes a state and returns another
 * state. The value stream from the original Observable is responsible for
 * providing values to the reducer.
 * @template T Generics parameter.
 * @param {Observable<ActionType<T>>} obs An Observable instance.
 * @param {Reducer<T>} reducer A Reducer instance.
 * @returns {Observable<RxReducer<T>>} An Observable instance.
 */
export declare function createReducer<T>(obs: Observable<ActionType<T>>, reducer: Reducer<T>): Observable<RxReducer<T>>;
/**
 * Create a Redux store that accepts an Array of reducer streams, then scan them
 * to derive the latest state.
 *
 * The setup should be as follows:
 *  - Define BehaviorSubject instances that accept state values.
 *  - Create reducer streams using createReducer.
 *  - Pass the resulting reducer streams to this method.
 *  - When a new state value should be updated, call next(value).
 *  - The new state will be calculated and pushed onto the store stream.
 * @template T Generics parameter.
 * @param {...Observable<RxReducer<T>>[]} reducers An Array of Observable.
 * @returns {Observable<stateinfo.Type<T>>} An Observable instance.
 */
export declare function create<T>(...reducers: Observable<RxReducer<T>>[]): Observable<stateinfo.Type<T>>;
/**
 * Represents a rx-based store.
 * @extends {StoreType} StoreType extension.
 */
export interface Type extends StoreType {
}
/**
 * This store is optional. It only provides some convenience when dealing with
 * state streams.
 * @implements {Type} Type implementation.
 */
export declare class Self implements Type {
    private store;
    constructor(...reducers: Observable<RxReducer<any>>[]);
    stateInfoStream(): Observable<stateinfo.Type<any>>;
    stateStream(): Observable<S.Type<any>>;
    valueAtNode(id: string): Observable<Try<any>>;
    stringAtNode(id: string): Observable<Try<string>>;
    numberAtNode(id: string): Observable<Try<number>>;
    booleanAtNode(id: string): Observable<Try<boolean>>;
}
