"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var javascriptutilities_1 = require("javascriptutilities");
var type_safe_state_js_1 = require("type-safe-state-js");
var Utils = require("./util");
/**
 * Convenience method to create an action with a default name.
 * @template T Generics parameter.
 * @param {ActionType<T>} action An ActionType instance.
 * @returns {action.Type<T>} An Action type instance.
 */
function createAction(action) {
    if (javascriptutilities_1.Types.isInstance(action, 'name', 'value')) {
        return action;
    }
    else {
        return { name: 'DummyAction', value: action };
    }
}
/**
 * Create reducer streams that can be used to calculate states. Beware that the
 * resulting Observable emits a function that takes a state and returns another
 * state. The value stream from the original Observable is responsible for
 * providing values to the reducer.
 * @template T Generics parameter.
 * @param {Observable<ActionType<T>>} obs An Observable instance.
 * @param {Reducer<T>} reducer A Reducer instance.
 * @returns {Observable<RxReducer<T>>} An Observable instance.
 */
function createReducer(obs, reducer) {
    return obs.pipe(operators_1.map(function (v) { return function (state) {
        var action = createAction(v);
        return { state: reducer(state, action), lastAction: createAction(action) };
    }; }));
}
exports.createReducer = createReducer;
/**
 * Create a Redux store that accepts an Array of reducer streams, then scan them
 * to derive the latest state.
 *
 * The setup should be as follows:
 *  - Define BehaviorSubject instances that accept state values.
 *  - Create reducer streams using createReducer.
 *  - Pass the resulting reducer streams to this method.
 *  - When a new state value should be updated, call next(value).
 *  - The new state will be calculated and pushed onto the store stream.
 * @template T Generics parameter.
 * @param {...Observable<RxReducer<T>>[]} reducers An Array of Observable.
 * @returns {Observable<stateinfo.Type<T>>} An Observable instance.
 */
function create() {
    var reducers = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        reducers[_i] = arguments[_i];
    }
    return rxjs_1.merge.apply(void 0, reducers).pipe(operators_1.scan(function (v1, v2) {
        return v2(v1.state);
    }, { state: type_safe_state_js_1.State.empty(), lastAction: undefined }), operators_1.startWith({ state: type_safe_state_js_1.State.empty(), lastAction: undefined }), operators_1.observeOn(rxjs_1.queueScheduler));
}
exports.create = create;
/**
 * This store is optional. It only provides some convenience when dealing with
 * state streams.
 * @implements {Type} Type implementation.
 */
var Self = /** @class */ (function () {
    function Self() {
        var reducers = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            reducers[_i] = arguments[_i];
        }
        this.store = create.apply(void 0, reducers);
    }
    Self.prototype.stateInfoStream = function () {
        return this.store;
    };
    Self.prototype.stateStream = function () {
        return this.store.pipe(operators_1.map(function (v) { return v.state; }));
    };
    Self.prototype.valueAtNode = function (id) {
        return Utils.valueAtNode(this.stateStream(), id);
    };
    Self.prototype.stringAtNode = function (id) {
        return Utils.stringAtNode(this.stateStream(), id);
    };
    Self.prototype.numberAtNode = function (id) {
        return Utils.numberAtNode(this.stateStream(), id);
    };
    Self.prototype.booleanAtNode = function (id) {
        return Utils.booleanAtNode(this.stateStream(), id);
    };
    return Self;
}());
exports.Self = Self;
//# sourceMappingURL=rx.js.map